' Sensor-Bug mit Entprellung
' Wenn man unmittelbar nach Reset (unterer Anschlag Poti) auf Maximum (oberer Anschlag Poti) dreht,
' wird der Sideswiper-Modus aktiviert. Zurueck in Bug-Mode nach neuem Reset

' Sprache: Bascom-AVR-Basic Version 2.0.7.9 Demo/Freeware
' Lizenz: WtfPL (https://de.wikipedia.org/wiki/WTFPL)

' - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2 MHz
$hwstack = 8                                                ' Platz fuer nur 4 Unterprogramme reserviert, spart SRAM
$swstack = 0                                                ' wird nicht gebraucht
$framesize = 0                                              ' dito

Dim C As Byte                                               ' gleitender Mittelwert fuer Dah-Erkennung
Dim D As Byte                                               ' gleitender Mittelwert fuer Dit-Erkennung
Dim Anadit As Word                                          ' ADC-Wert fuer Erkennung Dit-Sensor
Dim Anadah As Word
Dim Refana As Word                                          ' speichert nach Reset den Sensor-Schwellwert
Dim Offtimer As Byte                                        ' Zaehler fuer Abschaltautomatik
Dim Dit As Word                                             ' Punktlaenge
Dim Dit_cnt As Word                                         ' wird bis Punktlaenge gesetzt und dann runtergezaehlt
Dim Ditpart As Word                                         ' halbe Punktlaenge fuer simuliertes Einschwingen des Bug-Punkt-Pendels
Dim Pause As Byte                                           ' Pause=0 : Punkt wird gesendet; Pause=1 : Punktpause
Dim Bugmode As Byte

Acsr.acd = 1                                                ' Analog-Komparator ausschalten, spart etwas Strom

Config Adc = Single , Prescaler = Auto                      ' AD-Converter konfigurieren und starten
Start Adc

Ddrb = &B0000_0110                                          ' DDRB.0 ungenutzt; DDRB.1 Tast-Ausgang; DDRB.2 Sensor-ADC-Referenz
Portb = &B000_0000                                          ' PB1=1 erzeugt Punkt nach (Anzeige Reset-Ende)
Didr0 = &B00111111                                          ' Digitaleingaenge abschalten; spart Strom

On Watchdog Onwdt                                           ' Interrupt-Routine fuer Watchdog-interrupt

' Watchdog als Interrupt-Quelle konfigurieren -      WDCR:  7=WDTIF  6=WDIE  5=WDP3  4=WDCE  3=WDE  2=WDP2  1=WDP1  0=WDP0
Wdtcr = &B0001_1111                                         ' Schreiben auf WDTCR Vorbereiten
Wdtcr = &B0110_0001                                         ' Interrupt statt Reset, Watchdog- Timer auf 8 sec setzen
Sreg.7 = 1                                                  ' Interrupts global freigeben


Waitms 300                                                  ' Vorlaufzeit ADC; auch Wartezeit fuer Erkennung Sideswiper-Modus
Gosub DitSensor                                             ' einmal ADC-Leerlaufwert am Punktsensor messen (Anpassung an Sensor)
Refana = Anadit                                             ' Leerlauf-Wert
Shift Anadit, right, 2                                      ' 25%
Refana = Refana + Anadit                                    ' Refana 25% groesser als Leerlaufwert
Const Prelldauer =  16  ' 32                                ' obere Grenze fuer gleitenden Mittelwert
Const Schwelle = (Prelldauer / 2) + Prelldauer / 4          ' Umschaltschwelle
Gosub Adclesen

Do
    Do
        GoSub DahSensor                                     ' ADC-Wert abholen
        If Anadah > Refana Then
            Incr C                                          ' gleitenden Mittelwert erhoehen
        Else
            Decr C                                          ' gleitenden Mittelwert vermindern
        End If
        If C < 2 Then                                       ' Ueberlauf verhindern
            C = 1
        End If
        If C > Prelldauer Then                              ' Ueberlauf verhindern
            C = Prelldauer
        End If
        If C > Schwelle Then                                ' oberer Hysteresewert ueberschritten
            PortB.1 = 1
            Offtimer = 0                                    ' Abschalt-Timer zuruecksetzen
        Else                                                ' unterer Hysteresewert unterschritten
            PortB.1 = 0
        End If
    Loop Until C < Schwelle

    Do
        GoSub DitSensor
        If Anadit > Refana Then
            Incr D                                          ' gleitenden Mittelwert erhoehen
        Else
            Decr D                                          ' gleitenden Mittelwert vermindern
        End If
        If D < 2 Then                                       ' Ueberlauf verhindern
            D = 1
        End If
        If D > Prelldauer Then                              ' Ueberlauf verhindern
            D = Prelldauer
        End If
        If D > Schwelle Then                                ' oberer Hysteresewert ueberschritten
            If Bugmode = 1 Then                             ' Bug Mode
                If Dit_cnt <= 1 Then                        ' wenn Dit_cnt runtergezaehlt
                    Gosub ADClesen                          ' ggf. neue Speed abholen
                    Dit_cnt = Dit                           ' Dit_cnt wieder auf Maximum setzen
                    Toggle Pause                            ' jedes Mal zwischen Punkt und Pause zwischen Punkten umschalten
                End If
                If Pause = 0 Then                           ' Punkt soll rausgehen
                    PortB.1 = 1
                Else                                        ' also waehrend Punkt-Pause
                    PortB.1 = 0
                End If
                Decr Dit_cnt
            Else                                            ' Sideswiper-Modus
                PortB.1 = 1                                 ' bewirkt Dauerton statt Punkten
            End If
        End If
        If D < Schwelle Then                                ' unterer Hysteresewert unterschritten                                 ' unterer Hysteresewert unterschritten
            PortB.1 = 0
            Dit_cnt = Ditpart                               ' simuliert Einschwingen des Punkt-Pendels
            Pause = 255                                     ' uebler workaround, Teil des o.a. Einschwingens. 255 bewirkt,
            End If                                          '    dass Pause nur ein Mal auf ungleich 1 (entspr. 0) gesetzt wird
        Loop until D < Schwelle
        PortB.1 = 0
Loop


Dahsensor:
    Admux = 1                                               ' ADC1 (Referenz) an ADC-Eingang legen ; der muss auf Low liegen (bereits zu Anfang fest konfiguriert)
    Portb.3 = 1                                             ' Pullup an Punktsensor ein
    Ddrb.3 = 1                                              ' Punktsensor nun Ausgang  -->  Sensorpin laden
    Waitus 32
    Ddrb.3 = 0                                              ' Punktsensor wieder Eingang
    Portb.3 = 0                                             ' Pullup an Punktsensor aus --> Sensorpin floatet
    Anadah = Getadc(3)                                      ' Punktsensor auslesen
Return

Ditsensor:
    Admux = 1                                               ' ADC1 (Referenz) an ADC-Eingang legen
    Portb.4 = 1                                             ' Pullup an Strichsensor ein
    Ddrb.4 = 1                                              ' Strichsensor ist nun Ausgang
    Waitus 32
    Ddrb.4 = 0                                              ' Strichsensor wieder Eingang
    Portb.4 = 0                                             ' Pullup an Strichsensor aus
    Anadit = Getadc(2)                                      ' Strichsensor auslesen
Return


Adclesen:
    Dit = Getadc(0)                                         ' Poti an ADC1/PB22
    If Dit < 960 Then
        Bugmode = 1
    End If
    Shift Dit , Right, 2                                    ' Teilung / 4 -> Werte nun bis 255
    Dit = Dit + 32                                          ' nun bis 287
    Dit = 384 - Dit                                         ' nutzbarer Bereich nun: 15 - 35 WpM
    Ditpart = Dit                                           ' wird genutzt fuer fuer simuliertes Einschwingen des Punktpendels
    Shift Ditpart , Right, 2                                ' Teilung / 2
    Ditpart = Ditpart + 32
Return


Onwdt:
    Incr Offtimer                                           ' alle 8 sec wird Offtimer incrementiert
    If Offtimer > 126 Then                                  ' nach 127 * 8 sec. (17 Min) erfolgt Powerdown
        Stop Adc
        DDRB = 0
        Mcucr = &B0011_0000                                 ' Bit 4=1 und Bit 3=0: Powerdown     Bit 5=1: Sleep enable
        !Sleep                                              ' Assembler-Befehl statt Basic-Befehl "Powerdown"; spart ein paar Bytes
    End If
Return

End
